/*
 * Created by Drew Carlson.
 * Copyright (c) 2021 Breadwinner AG.  All right reserved.
 *
 * See the LICENSE file at the project root for license information.
 * See the CONTRIBUTORS file at the project root for a list of contributors.
 */
package com.blockset.walletkit

import kotlin.test.*


class NetworkTest {

    @Ignore // TODO: Missing builtin networks
    @Test
    fun testNetworkBTC() {
        val btc = Currency.create("bitcoin-mainnet:__native__", "Bitcoin", "btc", "native", null)

        val unitSat = UnitWK.create(btc, "sat", "Satoshi", "SAT")
        val unitBtc = UnitWK.create(btc, "btc", "Bitcoin", "B", unitSat, 8u)

        val network = checkNotNull(Network.findBuiltin("bitcoin-mainnet"))

        assertEquals("bitcoin-mainnet", network.uids)
        assertEquals("Bitcoin", network.name)
        assertTrue(network.isMainnet)

        val height = network.height
        network.height *= 2u
        assertEquals(height * 2u, network.height)


        assertEquals(btc, network.currency)
        assertTrue(network.hasCurrency(btc))
        assertEquals(btc, network.currencyByCode("btc"))
        assertNull(network.currencyByIssuer("foo"))

        assertEquals(unitSat, network.baseUnitFor(btc))
        assertEquals(unitBtc, network.defaultUnitFor(btc))

        assertEquals(setOf(unitSat, unitBtc), network.unitsFor(btc))
        assertTrue(network.hasUnitFor(btc, unitBtc) ?: false)
        assertTrue(network.hasUnitFor(btc, unitSat) ?: false)

        val eth = Currency.create("ethereum-mainnet:__native__", "Ethereum", "ETH", "native", null)
        val unitWei = UnitWK.create(eth, "ETH-WEI", "WEI", "wei")

        assertFalse(network.hasCurrency(eth))
        assertNull(network.baseUnitFor(eth))
        assertNull(network.unitsFor(eth))

        assertFalse(network.hasUnitFor(eth, unitWei) ?: false)
        assertFalse(network.hasUnitFor(eth, unitBtc) ?: false)
        assertFalse(network.hasUnitFor(btc, unitWei) ?: false)

        assertEquals(1, network.fees.size)

        val networksTable = mapOf(network to 1)
        assertEquals(1, networksTable[network])
    }

    @Ignore // TODO: Missing builtin networks
    @Test
    fun testNetworkETH() {
        val eth = Currency.create("ethereum-mainnet:__native__", "Ethereum", "eth", "native", null)
        val unitWei = UnitWK.create(eth, "wei", "WEI", "wei")
        val unitGwei = UnitWK.create(eth, "gwei", "GWEI", "gwei", unitWei, 9u)
        val unitEther = UnitWK.create(eth, "eth", "ETHER", "E", unitWei, 18u)

        val brd = Currency.create("ethereum-mainnet:0x558ec3152e2eb2174905cd19aea4e34a23de9ad6", "BRD Token", "brd", "erc20", "0x558ec3152e2eb2174905cd19aea4e34a23de9ad6")

        //val unitBrdi = CUnit.create(brd, "BRD_Integer", "BRD Integer", "BRDI")
        //val unitBrd = CUnit.create(brd, "BRD_Decimal", "BRD Decimal", "BRD", unitBrdi, 18u)

        val btc = Currency.create("bitcoin-mainnet:__native__", "Bitcoin", "btc", "native", null)

        val fee1 = NetworkFee(1u * 60u * 1000uL, Amount.create(2.0, unitGwei))

        val network = checkNotNull(Network.findBuiltin("ethereum-mainnet"))

        assertEquals("Ethereum", network.toString())
        assertTrue(network.hasCurrency(eth))
        assertTrue(network.hasCurrency(brd))
        assertFalse(network.hasCurrency(btc))

        assertNotNull(network.currencyByCode("eth"))
        assertNotNull(network.currencyByCode("brd"))

        assertNotNull(network.currencyByIssuer("0x558ec3152e2eb2174905cd19aea4e34a23de9ad6"))
        assertNotNull(network.currencyByIssuer("0x558ec3152e2eb2174905cd19aea4e34a23de9ad6".uppercase()))
        assertNull(network.currencyByIssuer("foo"))

        assertTrue(network.hasUnitFor(eth, unitWei) ?: false)
        assertTrue(network.hasUnitFor(eth, unitGwei) ?: false)
        assertTrue(network.hasUnitFor(eth, unitEther) ?: false)

        // TODO: Fails assertEquals(fee1, network.minimumFee)

        assertNull(network.defaultUnitFor(btc))
        assertNull(network.baseUnitFor(btc))
    }
}
